@@ -1,12 +1,13 @@
 """Vedirect init logger entry."""
 import logging
+from vedirect_m8.version import VERSION
 # Version of realpython-reader package
 __author__ = "Janne Kario, Eli Serra"
 __copyright__ = "Copyright 2015, Janne Kario"
 __deprecated__ = False
 __license__ = "MIT"
 __status__ = "Production"
-__version__ = "1.2.9.2"
+__version__ = VERSION
 
 
 class AppFilter(logging.Filter):
@@ -16,7 +17,7 @@ class AppFilter(logging.Filter):
 
     def filter(self, record):
         """Logger app version."""
-        record.app_version = "vedirect-%s" % __version__
+        record.app_version = f"vedirect-{__version__}"
         return True
 
 
@@ -35,7 +36,8 @@ def configure_logging(debug: bool = False):
     logger.setLevel(logging.INFO)
 
     formatter = logging.Formatter(
-        '%(asctime)s.%(msecs)03d :: %(app_version)s :: %(message)s', "%Y-%m-%d %H:%M:%S"
+        '%(asctime)s.%(msecs)03d :: %(app_version)s :: %(message)s',
+        "%Y-%m-%d %H:%M:%S"
     )
     syslog.setFormatter(formatter)
 
