@@ -4,24 +4,26 @@ Connect to serial port.
 
 Contain method to scan all available ports.
 
- .. seealso: Vedirect, VedirectController
- .. raises: SerialException,
-             SerialTimeoutException
+ .. see also: Vedirect, VedirectController
+ Raise:
+  - SerialConfException,
+  - OpenSerialVeException
+  - SerialVeException
 """
 import logging
 import os
-from serial import Serial, SerialException, SerialTimeoutException
+from typing import Optional, Union
+from serial import Serial, SerialException
 import serial.tools.list_ports as serial_list_ports
 from ve_utils.usys import USys
 from vedirect_m8.serutils import SerialUtils as Ut
-from vedirect_m8.exceptions import VedirectException
+from vedirect_m8.exceptions import SerialConfException
+from vedirect_m8.exceptions import OpenSerialVeException
+from vedirect_m8.exceptions import SerialVeException
 
 __author__ = "Eli Serra"
 __copyright__ = "Copyright 2020, Eli Serra"
-__deprecated__ = False
 __license__ = "MIT"
-__status__ = "Production"
-__version__ = "2.0.0"
 
 logging.basicConfig()
 logger = logging.getLogger("vedirect")
@@ -42,23 +44,24 @@ class SerialConnection:
             - Flow Control None
     """
     def __init__(self,
-                 serial_port: str or None = None,
+                 serial_port: Optional[str] = None,
                  baud: int = 19200,
-                 timeout: int or float = 0,
+                 timeout: Union[int, float] = 0,
                  source_name: str = 'void'
                  ):
         """
         Constructor of SerialConnection class.
 
         :Example:
-            - > sc = SerialConnection(serial_port = "/dev/ttyUSB1")
-            - > sc.connect()
-            - > True # if connection opened on serial port "/dev/tyyUSB1"
+            >>> sc = SerialConnection(serial_port = "/dev/ttyUSB1")
+            >>> sc.connect()
+            >>> True # if connection opened on serial port "/dev/tyyUSB1"
         :param self: Refer to the object instance itself,
         :param serial_port: The serial port to connect,
         :param baud: Baud rate such as 9600 or 115200 etc.
         :param timeout: Set a read timeout value in seconds,
-        :param source_name: This is used in logger to identify the source of call.
+        :param source_name: This is used in logger
+         to identify the source of call.
         :return: Nothing
         """
         self._source_name = 'void'
@@ -71,17 +74,17 @@ class SerialConnection:
                             timeout=timeout,
                             source_name=source_name)
 
-    def get_serial_port(self) -> int or float:
+    def get_serial_port(self) -> Optional[str]:
         """Return serial_port value from instance."""
         return self._serial_port
 
-    def set_serial_port(self, serial_port: str or None = None) -> bool:
+    def set_serial_port(self, serial_port: Optional[str]) -> bool:
         """
         Set serial_port value.
 
         :Example :
-            - >self.set_serial_port(serial_port="/tmp/vmodem0")
-            - >True
+            >>> self.set_serial_port(serial_port="/tmp/vmodem0")
+            >>> True
         :param serial_port: The serial port to set.
         :return: True if is valid serial_port
         """
@@ -90,13 +93,13 @@ class SerialConnection:
             return True
         return False
 
-    def set_baud(self, baud: int or None = None) -> bool:
+    def set_baud(self, baud: Optional[int]) -> bool:
         """
         Set baud value.
 
         :Example :
-            - >self.set_baud(baud=19200)
-            - >True
+            >>> self.set_baud(baud=19200)
+            >>> True
         :param baud: The baud to set.
         :return: True if is valid baud
         """
@@ -105,17 +108,17 @@ class SerialConnection:
             return True
         return False
 
-    def get_timeout(self) -> int or float:
+    def get_timeout(self) -> Union[int, float]:
         """Return timeout value from instance."""
         return self._timeout
 
-    def set_timeout(self, timeout: int or float or None = None) -> bool:
+    def set_timeout(self, timeout: Optional[Union[int, float]]) -> bool:
         """
         Set timeout value.
 
         :Example :
-            - >self.set_timeout(timeout=0)
-            - >True
+            >>> self.set_timeout(timeout=0)
+            >>> True
         :param timeout: The timeout to set.
         :return: True if is valid timeout
         """
@@ -124,15 +127,15 @@ class SerialConnection:
             return True
         return False
 
-    def set_source_name(self, source_name: str or None = None) -> bool:
+    def set_source_name(self, source_name: Optional[str]) -> bool:
         """
         Set source_name value.
 
         This is used in logger to identify the source of call.
         When multiple serial connections are defined.
         :Example :
-            - >self.set_source_name(source_name='BMV700')
-            - >True
+            >>> self.set_source_name(source_name='BMV700')
+            >>> True
         :param source_name: The source_name to set.
         :return: True if is valid source_name
         """
@@ -145,7 +148,9 @@ class SerialConnection:
         """
         Test if the object is ready.
 
-        :return: True if configuration settings are valid and if the serial connection is opened.
+        :return:
+            True if configuration settings are valid
+            and if the serial connection is opened.
         """
         return self.is_settings() and self.is_serial_ready()
 
@@ -153,7 +158,9 @@ class SerialConnection:
         """
         Test if is serial connection is ready.
 
-        :return: True if self._ser is an instance of Serial and if the serial connection is opened.
+        :return:
+            True if self._ser is an instance of Serial
+            and if the serial connection is opened.
         """
         return isinstance(self.ser, Serial) and self.ser.isOpen()
 
@@ -168,27 +175,27 @@ class SerialConnection:
                                                timeout=self._timeout)
 
     def _init_settings(self,
-                       serial_port: str or None = None,
+                       serial_port: Optional[str] = None,
                        baud: int = 19200,
-                       timeout: int or float or None = 0,
+                       timeout: Optional[Union[int, float]] = 0,
                        source_name: str = 'void'
                        ) -> bool:
         """
         Initialise configuration settings.
 
         :Example :
-            - > self.init_settings( \r
-                                  serial_port="/tmp/vmodem0", \r
-                                  baud=19200, \r
-                                  timeout=0, \r
-                                  source_name="BMV700" \r
-                                  ) \r
-            - > True
-        :param serial_port: The serial port,
-        :param baud: The baudrate.
-        :param timeout: The timeout.
+            >>> self._init_settings(
+            >>>     serial_port="/tmp/vmodem0",
+            >>>     baud=19200,
+            >>>     timeout=0,
+            >>>     source_name="BMV700"
+            >>> )
+            >>> True
+        :param serial_port: The serial port
+        :param baud: The serial baud rate
+        :param timeout: The serial timeout
         :param source_name: The source_name.
-        :return: True if configuration settings are valid
+        :return: True if configuration settings are valid.
         """
         self.set_serial_port(serial_port)
         self.set_baud(baud)
@@ -197,70 +204,98 @@ class SerialConnection:
         return self.is_settings()
 
     def _set_serial_conf(self,
-                         serial_port: str or None = None,
-                         baud: int or None = None,
-                         timeout: int or float or None = -1,
-                         write_timeout: int or float or None = -1,
+                         serial_port: Optional[str] = "default",
+                         baud: Optional[int] = None,
+                         timeout: Optional[Union[int, float]] = -1,
+                         write_timeout: Optional[Union[int, float]] = -1,
                          exclusive: bool = False
-                         ) -> dict:
+                         ) -> Optional[dict]:
         """
-        Return the serial configuration settings to open serial connection.
+        Set serial configuration settings to open serial connection.
 
         :Example :
-            - > self.set_serial_conf( \r
-                                  serial_port="/tmp/vmodem0", \r
-                                  baud=19200, \r
-                                  timeout=0, \r
-                                  write_timeout=0, \r
-                                  exclusive=True \r
-                                  ) \r
-            - > True
-        :param serial_port: The serial port,
-        :param baud: The baudrate.
-        :param timeout: The timeout.
-        :param write_timeout: The write_timeout,
-        :param exclusive: The exclusive.
-        :return: a dictionary with the configuration to open a serial connection.
+            >>> self._set_serial_conf(
+            >>>     serial_port="/tmp/vmodem0",
+            >>>     baud=19200,
+            >>>     timeout=0,
+            >>>     write_timeout=0,
+            >>>     exclusive=True
+            >>> )
+            >>> True
+        :param serial_port: The serial port
+        :param baud: The serial baud rate
+        :param timeout: The serial timeout
+        :param write_timeout: The serial write timeout
+        :param exclusive: Set exclusive access mode (POSIX only).
+            A port cannot be opened in exclusive access mode
+            if it is already open in exclusive access mode.
+        :return:
+            a dictionary with the configuration to open a serial connection.
+            Or None if a parameter is invalid.
         """
-        self.set_serial_port(serial_port)
-        self.set_baud(baud)
-        self.set_timeout(timeout)
-        result = {
-            'port': self._serial_port,
-            'baudrate': self._baud,
-            'timeout': self._timeout
-        }
-        if SerialConnection.is_timeout(write_timeout):
-            result.update({'write_timeout': write_timeout})
-        if Ut.str_to_bool(exclusive) is True:
-            result.update({'exclusive': True})
+        if (serial_port != "default"
+                and not SerialConnection.is_serial_port(serial_port))\
+                or (baud is not None
+                    and not SerialConnection.is_baud(baud))\
+                or (timeout != -1
+                    and not SerialConnection.is_timeout(timeout))\
+                or (write_timeout != -1
+                    and not SerialConnection.is_timeout(write_timeout)):
+            result = None
+        else:
+            self.set_serial_port(serial_port)
+            self.set_baud(baud)
+            self.set_timeout(timeout)
+
+            result = {
+                'port': self._serial_port,
+                'baudrate': self._baud,
+                'timeout': self._timeout
+            }
+            if SerialConnection.is_timeout(write_timeout):
+                result.update({'write_timeout': write_timeout})
+            if Ut.str_to_bool(exclusive) is True:
+                result.update({'exclusive': True})
         return result
 
     def connect(self,
-                serial_port: str or None = None,
-                baud: int or None = None,
-                timeout: int or float or None = None,
-                write_timeout: int or float or None = None,
+                serial_port: Optional[str] = "default",
+                baud: Optional[int] = None,
+                timeout: Optional[Union[int, float]] = -1,
+                write_timeout: Optional[Union[int, float]] = -1,
                 exclusive: bool = False
                 ) -> bool:
         """
         Start serial connection from parameters.
 
         :Example :
-            - > self.connect( \r
-                                  serial_port="/tmp/vmodem0", \r
-                                  baud=19200, \r
-                                  timeout=0, \r
-                                  write_timeout=0, \r
-                                  exclusive=True \r
-                                  ) \r
-            - > True
-        :param serial_port: The serial port,
-        :param baud: The baudrate.
-        :param timeout: The timeout.
-        :param write_timeout: The write_timeout,
-        :param exclusive: The exclusive.
+            >>> self.connect(
+            >>>     serial_port="/tmp/vmodem0",
+            >>>     baud=19200,
+            >>>     timeout=0,
+            >>>     write_timeout=0,
+            >>>     exclusive=True
+            >>> )
+            >>> True
+
+        :param serial_port: The serial port
+        :param baud: The serial baud rate
+        :param timeout: The serial timeout
+        :param write_timeout: The serial write timeout
+        :param exclusive: Set exclusive access mode (POSIX only).
+            A port cannot be opened in exclusive access mode
+            if it is already open in exclusive access mode.
         :return: True if success to open a serial connection.
+        Raise:
+         - SerialConfException:
+           Will be raised when parameter
+           are out of range or invalid,
+           e.g. serial_port, baud rate, data bits
+         - SerialVeException:
+           In case the device can not be found or can not be configured.
+         - OpenSerialVeException:
+           Will be raised when the device is configured
+           but port is not opened.
         """
         serial_conf = self._set_serial_conf(
             serial_port=serial_port,
@@ -274,43 +309,42 @@ class SerialConnection:
             'settings : %s',
             self._source_name, serial_conf
         )
-        if self.is_settings():
+        if SerialConnection.is_serial_conf_data(serial_conf):
             try:
                 self.ser = Serial(**serial_conf)
                 if self.ser.isOpen():
                     logger.info(
-                        '[SerialConnection::connect::%s] New Serial connection established. '
+                        '[SerialConnection::connect::%s] '
+                        'New Serial connection established. '
                         'args : %s.',
                         self._source_name, serial_conf
                     )
                     result = True
                 else:
-                    raise VedirectException(
-                        '[SerialConnection::connect::%s] '
-                        'Unable to open serial connection. args: %s' %
-                        (self._source_name, serial_conf)
+                    raise OpenSerialVeException(
+                        f'[SerialConnection::connect::{self._source_name}] '
+                        'Unable to open serial connection. args: '
+                        f'{serial_conf}'
                     )
-            except SerialTimeoutException as ex:
-                raise VedirectException(
-                    '[SerialConnection::connect::%s] '
-                    'Exception when attempting to open serial connection. '
-                    ' args: %s - ex : %s' %
-                    (self._source_name, serial_conf, ex)
-                ) from SerialTimeoutException
 
             except SerialException as ex:
-                raise VedirectException(
-                    '[SerialConnection::connect::%s] '
+                raise SerialVeException(
+                    f'[SerialConnection::connect::{self._source_name}] '
                     'Exception when attempting to open serial connection. '
-                    ' args: %s - ex : %s' %
-                    (self._source_name, serial_conf, ex)
+                    f' args: {serial_conf} - ex : {ex}'
                 ) from SerialException
+            except ValueError as ex:
+                raise SerialConfException(
+                    f'[SerialConnection::connect::{self._source_name}] '
+                    'Parameter are out of range, e.g. baud rate, data bits. '
+                    f' args: {serial_conf} - ex : {ex}'
+                ) from ValueError
+
         else:
-            raise VedirectException(
-                '[SerialConnection::connect::%s] '
+            raise SerialConfException(
+                f'[SerialConnection::connect::{self._source_name}] '
                 'Unable to open serial connection. '
-                'Invalid settings : %s' %
-                (self._source_name, serial_conf)
+                f'Invalid configuration : {serial_conf}'
             )
 
         return result
@@ -320,15 +354,17 @@ class SerialConnection:
         Get all available ports on the machine.
 
         First get available virtual serial ports on /tmp/ directory.
-        Then use serial.tools.list_ports.comports() to get available serial ports.
+        Then use serial.tools.list_ports.comports()
+        to get available serial ports.
 
         :Example :
-            - >self.get_unix_virtual_serial_ports_list()
-            - >['/tmp/vmodem0', '/tmp/vmodem1', '/dev/ttyUSB1']
+            >>> self.get_serial_ports_list()
+            >>> ['/tmp/vmodem0', '/tmp/vmodem1', '/dev/ttyUSB1']
         :return: List of serial ports and virtual serial ports available.
         """
-        result = list()
+        result = []
         try:
+            # scan unix virtual serial ports ports
             result = self.get_unix_virtual_serial_ports_list()
             ports = serial_list_ports.comports()
             for port, desc, hwid in sorted(ports):
@@ -358,12 +394,12 @@ class SerialConnection:
             - vmodem[0-999]
 
         :Example :
-            - >self.get_unix_virtual_serial_ports_list()
-            - >['/tmp/vmodem0', '/tmp/vmodem1']
+            >>> self.get_unix_virtual_serial_ports_list()
+            >>> ['/tmp/vmodem0', '/tmp/vmodem1']
 
         :return: List of virtual serial ports available.
         """
-        result = list()
+        result = []
         try:
             if USys.is_op_sys_type('unix'):
                 for path in SerialConnection._get_virtual_ports_paths():
@@ -379,13 +415,28 @@ class SerialConnection:
             )
         return result
 
+    def serialize(self):
+        """
+        This method allows to serialize in a proper way this object
+
+        :return: A dict of order
+        :rtype: Dict
+        """
+
+        return {
+            'source_name': self._source_name,
+            'serial_port': self._serial_port,
+            'baud': self._baud,
+            'timeout': self._timeout
+        }
+
     @staticmethod
     def _scan_path(path: str) -> list:
         """Scan path and get serial ports from it."""
         result = None
         if path != "/dev" and os.path.exists(path):
             # get list files from path
-            result = list()
+            result = []
             for entry in os.scandir(path):
                 if not os.path.isdir(entry.path) \
                         and Ut.is_serial_port_name_pattern(entry.name):
@@ -394,7 +445,7 @@ class SerialConnection:
         return result
 
     @staticmethod
-    def get_default_serial_conf(conf: dict or None) -> dict:
+    def get_default_serial_conf(conf: Optional[dict]) -> dict:
         """Get serial configuration data with default values."""
         result = {
             "serial_port": None,
@@ -402,21 +453,28 @@ class SerialConnection:
             "timeout": 0
         }
         if Ut.is_dict(conf, not_null=True):
-            serial_conf = Ut.get_items_from_dict(conf, ["serial_port", "baud", "timeout"])
+            serial_conf = Ut.get_items_from_dict(
+                data=conf,
+                list_keys=["serial_port", "baud", "timeout"]
+            )
             if Ut.is_dict(serial_conf, not_null=True):
                 result.update(serial_conf)
         return result
 
     @staticmethod
-    def is_serial_conf(serial_port: str,
+    def is_serial_conf(serial_port: Optional[str],
                        baud: int,
-                       timeout: int or float) -> bool:
+                       timeout: Union[int, float]) -> bool:
         """
         Test if valid serial configuration settings.
 
         :Example :
-            - >self.is_serial_conf(serial_port="/tmp/vmodem0", baud=19200, timeout=0)
-            - >True
+            >>> SerialConnection.is_serial_conf(
+            >>>     serial_port="/tmp/vmodem0",
+            >>>     baud=19200,
+            >>>     timeout=0
+            >>> )
+            >>> True
         :param serial_port: The serial port,
         :param baud: The baudrate.
         :param timeout: The timeout.
@@ -426,6 +484,29 @@ class SerialConnection:
             and SerialConnection.is_baud(baud) \
             and SerialConnection.is_timeout(timeout)
 
+    @staticmethod
+    def is_serial_conf_data(conf: Optional[dict]) -> bool:
+        """
+        Test if valid serial configuration settings.
+
+        :Example :
+            >>> SerialConnection.is_serial_conf(
+            >>>     serial_port="/tmp/vmodem0",
+            >>>     baud=19200,
+            >>>     timeout=0
+            >>> )
+            >>> True
+        :param conf: The Configuration data
+        :return: True if configuration settings are valid
+        """
+        return Ut.is_dict(conf)\
+            and SerialConnection.is_serial_port(
+                conf.get('serial_port')) \
+            and SerialConnection.is_baud(
+                conf.get('baudrate')) \
+            and SerialConnection.is_timeout(
+                conf.get('timeout'))
+
     @staticmethod
     def _get_virtual_ports_paths() -> list:
         """
@@ -436,13 +517,15 @@ class SerialConnection:
         return [os.path.expanduser('~')]
 
     @staticmethod
-    def get_virtual_home_serial_port(port: str) -> str or None:
+    def get_virtual_home_serial_port(port: str) -> Optional[str]:
         """
         Return the virtual serial port path from user home directory.
 
         :Example :
-            - >SerialConnection.get_virtual_home_serial_port(port="vmodem0")
-            - >/home/${USER}/vmodem0
+            >>> SerialConnection.get_virtual_home_serial_port(
+            >>>     port="vmodem0"
+            >>> )
+            >>> "/home/${USER}/vmodem0"
         :param port: The port name to join at user home directory.
         :return: the virtual serial port path from user home directory.
                  Or None if port is invalid virtual port name.
@@ -460,10 +543,14 @@ class SerialConnection:
         First split the serial port in path and port name.
         Then test if serial port is a string, if path and port name are valid.
         :Example :
-            - >SerialConnection.is_virtual_serial_port(serial_port="/tmp/vmodem0")
-            - >True
-            - >SerialConnection.is_virtual_serial_port(serial_port="/run/vmodem0")
-            - >False
+            >>> SerialConnection._is_virtual_serial_port(
+            >>>     serial_port="/tmp/vmodem0"
+            >>> )
+            >>> True
+            >>> SerialConnection._is_virtual_serial_port(
+            >>>     serial_port="/run/vmodem0"
+            >>> )
+            >>> False
         :param serial_port: The serial port to test.
         :return: True if the virtual serial port is valid.
         """
@@ -478,8 +565,10 @@ class SerialConnection:
         Return serial port split in name and path.
 
         :Example :
-            - >SerialConnection.split_serial_port(serial_port="/tmp/vmodem0")
-            - >('vmodem0', '/tmp')
+            >>> SerialConnection._split_serial_port(
+            >>>     serial_port="/tmp/vmodem0"
+            >>> )
+            >>> ('vmodem0', '/tmp')
         :param serial_port: The serial port to split.
         :return: Tuple of name and path.
         """
@@ -494,8 +583,10 @@ class SerialConnection:
         Test serial_port path exists.
 
         :Example :
-            - >SerialConnection.is_serial_port_exists(serial_port="/tmp/vmodem0")
-            - >True
+            >>> SerialConnection.is_serial_port_exists(
+            >>>     serial_port="/tmp/vmodem0"
+            >>> )
+            >>> True
         :param serial_port: The serial port to test.
         :return: True if the serial port path exists and is a string instance.
         """
@@ -507,10 +598,12 @@ class SerialConnection:
         Test if is valid baud rate.
 
         :Example :
-            - >SerialConnection.is_baud(baud=1200)
-            - >True
+            >>> SerialConnection.is_baud(baud=1200)
+            >>> True
         :param baud: The baudrate value to test.
-        :return: True if the baudrate value is valid and is a integer instance.
+        :return:
+            True if the baudrate value is valid
+            and is an integer instance.
         """
         return Ut.is_int(baud)\
             and baud in [
@@ -521,46 +614,56 @@ class SerialConnection:
            ]
 
     @staticmethod
-    def is_timeout(timeout: int or float) -> bool:
+    def is_timeout(timeout: Optional[Union[int, float]]) -> bool:
         """
         Test if is valid serial read timeout.
 
-        Possible values for the parameter timeout which controls the behavior of read():
-         - timeout = None: wait forever / until requested number of bytes are received.
-         - timeout = 0: non-blocking mode, return immediately in any case,
-           returning zero or more, up to the requested number of bytes
-         - timeout = x: set timeout to x seconds (float allowed)
-           returns immediately when the requested number of bytes are available,
-           otherwise wait until the timeout expires
-           and return all bytes that were received until then.
+        Possible values for the parameter timeout
+        which controls the behavior of read():
+         - timeout = None:
+            wait forever / until requested number of bytes are received.
+         - timeout = 0:
+            non-blocking mode, return immediately in any case,
+            returning zero or more, up to the requested number of bytes
+         - timeout = x:
+            set timeout to x seconds (float allowed)
+            returns immediately when the requested
+            number of bytes are available,
+            otherwise wait until the timeout expires
+            and return all bytes that were received until then.
 
         :Example :
-            - >SerialConnection.is_timeout(timeout=0)
-            - >True
+            >>> SerialConnection.is_timeout(timeout=0)
+            >>> True
         :param timeout: The timeout value to test.
         :return: True if is valid read timeout value.
         """
         return (Ut.is_numeric(timeout) and timeout >= 0) or timeout is None
 
     @staticmethod
-    def is_serial_port(serial_port: str) -> bool:
+    def is_serial_port(serial_port: Optional[str]) -> bool:
         """
         Test if is valid serial port.
 
         First split the serial port in path and port name.
         Then test if serial port is a string, if path and port name are valid.
         :Example :
-            - >SerialConnection.is_serial_port(serial_port="/tmp/vmodem0")
-            - >True
-            - >SerialConnection.is_serial_port(serial_port="/run/vmodem0")
-            - >False
+            >>> SerialConnection.is_serial_port(
+            >>>     serial_port="/tmp/vmodem0"
+            >>> )
+            >>> True
+            >>> SerialConnection.is_serial_port(
+            >>>     serial_port="/run/vmodem0"
+            >>> )
+            >>> False
         :param serial_port: The serial_port to test.
-        :return: True if the serial port is valid.
+        :return: True if the serial port is valid or None.
         """
         name, path = SerialConnection._split_serial_port(serial_port)
-        return Ut.is_str(serial_port)\
-            and SerialConnection.is_serial_path(path)\
-            and Ut.is_serial_port_name_pattern(name)
+        return (Ut.is_str(serial_port, not_null=True)
+                and SerialConnection.is_serial_path(path)
+                and Ut.is_serial_port_name_pattern(name)) \
+            or serial_port is None
 
     @staticmethod
     def is_serial_path(path: str) -> bool:
@@ -568,12 +671,14 @@ class SerialConnection:
         Test if is valid serial path.
 
         For win32 systems path must be null (None or "").
-        For unix systems path must be /dev or in get_virtual_ports_paths() method.
+        For unix systems path must be
+            - /dev
+            - or in get_virtual_ports_paths() method.
         :Example :
-            - >SerialConnection.is_serial_path(serial_port="/tmp")
-            - >True
-            - >SerialConnection.is_serial_path(serial_port="/run")
-            - >False
+            >>> SerialConnection.is_serial_path(serial_port="/tmp")
+            >>> True
+            >>> SerialConnection.is_serial_path(serial_port="/run")
+            >>> False
         :param path: The serial port path to test.
         :return: True if the serial port path is valid.
         """
