@@ -6,15 +6,13 @@ Test decoded serial vedirect data to identify a valid serial port.
  .. raises: SettingInvalidException
 """
 import logging
+from typing import Optional
 from vedirect_m8.serutils import SerialUtils as Ut
 from vedirect_m8.exceptions import SettingInvalidException
 
 __author__ = "Eli Serra"
 __copyright__ = "Copyright 2020, Eli Serra"
-__deprecated__ = False
 __license__ = "MIT"
-__status__ = "Production"
-__version__ = "1.0.0"
 
 logging.basicConfig()
 logger = logging.getLogger("vedirect")
@@ -39,7 +37,7 @@ class SerialTestHelper:
      .. seealso: VedirectController
      .. raises: SettingInvalidException
     """
-    def __init__(self, serial_tests: dict or None = None):
+    def __init__(self, serial_tests: Optional[dict] = None):
         """
         Constructor of SerialTestHelper class.
         :param serial_tests: The serialTests configuration settings
@@ -82,8 +80,10 @@ class SerialTestHelper:
         Run value test on serial_data.
 
         Evaluates if the value_test is valid,
-        if serial_data is a dictionary, and if serial_data contain a key value_test key value.
-        Then evaluate if serial_data key value is equal to the value_test value.
+        if serial_data is a dictionary,
+        and if serial_data contain a key value_test key value.
+        Then evaluate if serial_data key value is equal
+        to the value_test value.
 
         :Example :
             >>> SerialTestHelper.run_value_test(
@@ -94,8 +94,10 @@ class SerialTestHelper:
                 },\n
                 serial_data = {"PID": "0x204"})
             >>> True
-        :param value_test: The value test to evaluate on serial_data.
-        :param serial_data: The serial data decoded from serial vedirect device.
+        :param value_test:
+            The value test to evaluate on serial_data.
+        :param serial_data:
+            The serial data decoded from serial vedirect device.
         :return: True if value test success on serial_data.
         """
         if SerialTestHelper.is_value_test(value_test)\
@@ -180,6 +182,7 @@ class SerialTestHelper:
         tst = False
         if Ut.is_dict(data, not_null=True):
             tst = True
+            valid_type_tests = SerialTestHelper.get_valid_type_tests()
             for key, item in data.items():
                 if Ut.is_key_pattern(key)\
                         and Ut.is_dict(item, not_null=True):
@@ -190,22 +193,22 @@ class SerialTestHelper:
                             and not SerialTestHelper.is_value_test(item):
                         tst = False
                     elif type_test == "columns"\
-                            and not SerialTestHelper.is_columns_list_test(item):
+                            and not SerialTestHelper.is_columns_list_test(
+                                item):
                         tst = False
-                    elif type_test not in SerialTestHelper.get_valid_type_tests():
+                    elif type_test not in valid_type_tests:
                         raise SettingInvalidException(
                             "[SerialTestHelper::validate_serial_tests] "
-                            "unrecognized typeTest %s, from key %s."
-                            "typeTest must be in : %s" %
-                            (type_test, key, SerialTestHelper.get_valid_type_tests())
+                            f"unrecognized typeTest {type_test}, "
+                            f"from key {key}."
+                            f"typeTest must be in : {valid_type_tests}"
                         )
                 else:
                     raise SettingInvalidException(
                             "[SerialTestHelper::validate_serial_tests] "
-                            "invalid serialTest settings from key %s. "
+                            f"invalid serialTest settings from key {key}. "
                             "Key must start by char [a-zA-Z0-9], "
-                            "and must contain only chars [a-zA-Z0-9_]." %
-                            key
+                            "and must contain only chars [a-zA-Z0-9_]."
                         )
         return tst
 
@@ -248,10 +251,14 @@ class SerialTestHelper:
                 type_test = item.get("typeTest")
 
                 if type_test == "value"\
-                        and not SerialTestHelper.run_value_test(item, serial_data):
+                        and not SerialTestHelper.run_value_test(
+                            value_test=item,
+                            serial_data=serial_data):
                     tst = False
 
                 elif type_test == "columns"\
-                        and not SerialTestHelper.run_columns_test(item, serial_data):
+                        and not SerialTestHelper.run_columns_test(
+                            columns_test=item,
+                            serial_data=serial_data):
                     tst = False
         return tst
