@@ -6,25 +6,32 @@ This is a forked version of script originally created by Janne Kario.
 (https://github.com/karioja/vedirect).
 
  Raise:
-    - InputReadException,
-    - serial.SerialException,
-    - serial.SerialTimeoutException
+        - SettingInvalidException
+        - InputReadException
+        - PacketReadException
+        - ReadTimeoutException
+        - SerialConnectionException
+        - SerialConfException
+        - SerialVeException
+        - OpenSerialVeException
 """
 import logging
 import time
+from typing import Optional, Union
 from serial import SerialException
 from ve_utils.utype import UType as Ut
 from vedirect_m8.serconnect import SerialConnection
 from vedirect_m8.exceptions import SettingInvalidException
 from vedirect_m8.exceptions import InputReadException
-from vedirect_m8.exceptions import TimeoutException, VedirectException
+from vedirect_m8.exceptions import PacketReadException
+from vedirect_m8.exceptions import ReadTimeoutException
+from vedirect_m8.exceptions import SerialConnectionException
+from vedirect_m8.exceptions import SerialConfException
+from vedirect_m8.exceptions import SerialVeException
 
 __author__ = "Janne Kario, Eli Serra"
 __copyright__ = "Copyright 2015, Janne Kario"
-__deprecated__ = False
 __license__ = "MIT"
-__status__ = "Production"
-__version__ = "1.0.0"
 
 logging.basicConfig()
 logger = logging.getLogger("vedirect")
@@ -38,31 +45,44 @@ class Vedirect:
     (https://github.com/karioja/vedirect).
 
     :Example:
-            >>> sc = Vedirect({"serial_port": "/dev/ttyUSB1"})
-            >>> sc.read_data_single()
-            >>> {"ser_key1": "ser_value1", ...}
+        >>> sc = Vedirect(
+        >>>     serial_conf={"serial_port": "/dev/ttyUSB1"}
+        >>> )
+        >>> sc.read_data_single()
+        >>> {"ser_key1": "ser_value1", ...}
 
     Raise:
-        - InputReadException,
-        - serial.SerialException,
-        - serial.SerialTimeoutException
+        - SettingInvalidException
+        - InputReadException
+        - PacketReadException
+        - ReadTimeoutException
+        - SerialConnectionException
+        - SerialConfException
+        - SerialVeException
+        - OpenSerialVeException
     """
     def __init__(self,
                  serial_conf: dict,
                  source_name: str = 'Vedirect',
-                 auto_start: bool = True
+                 auto_start: bool = True,
+                 max_packet_blocks: Optional[int] = 18
                  ):
         """
         Constructor of Vedirect class.
 
         :Example:
-            >>> sc = Vedirect({"serial_port": "/dev/ttyUSB1"})
+            >>> sc = Vedirect(
+            >>>     serial_conf={"serial_port": "/dev/ttyUSB1"}
+            >>> )
             >>> sc.read_data_single()
             >>> {"ser_key1": "ser_value1", ...}
         :param self: Refer to the object instance itself,
-        :param serial_conf: dict: The serial connection configuration,
-        :param source_name: This is used in logger to identify the source of call,
-        :param auto_start: bool: Define if serial connection must be established automatically.
+        :param serial_conf: dict:
+            The serial connection configuration,
+        :param source_name:
+            This is used in logger to identify the source of call,
+        :param auto_start: bool:
+            Define if serial connection must be established automatically.
         :return: Nothing
         """
         self._com = None
@@ -72,12 +92,13 @@ class Vedirect:
             "hexmarker": ord(':'),
             "delimiter": ord('\t')
         }
+        self.max_blocks = 18
+        self.set_max_packet_blocks(max_packet_blocks)
         self.key = ''
         self.value = ''
         self.bytes_sum = 0
         self.state = self.WAIT_HEADER
         self.dict = {}
-        self.time_packet = 0
         self.init_settings(serial_conf=serial_conf,
                            source_name=source_name,
                            auto_start=auto_start
@@ -85,8 +106,46 @@ class Vedirect:
 
     (HEX, WAIT_HEADER, IN_KEY, IN_VALUE, IN_CHECKSUM) = range(5)
 
+    def has_free_block(self) -> bool:
+        """Test if block of key value can be added to dict packet."""
+        return self.max_blocks is None\
+            or len(self.dict) <= self.max_blocks
+
+    def set_max_packet_blocks(self, value: Optional[int]) -> bool:
+        """
+        Set max blocks by packet value.
+
+        If input packet in dict has more key that max_blocks,
+        raise a PacketReadException on reading serial.
+        To disable this limitation, set the value to None.
+        By default, the value is 18 blocks per packet.
+        (See VeDirect protocol)
+
+        Example :
+            >>> self.set_max_packet_blocks(22)
+            >>> True
+        :param value: max blocks by packet value
+        :return: True if max blocks by packet value is updated.
+
+        Raise:
+         - SettingInvalidException: if the value is invalid.
+        """
+        result = True
+        self.max_blocks = 18
+        if Ut.is_int(value, positive=True):
+            self.max_blocks = value
+        elif value is None:
+            self.max_blocks = None
+        else:
+            raise SettingInvalidException(
+                "[Vedirect:set_max_packet_blocks] "
+                "Max blocks by packet value is not valid. "
+                "Value must be integer or null. "
+            )
+        return result
+
     def has_serial_com(self) -> bool:
-        """Test if self._com is a valid SerialConnection instance."""
+        """Test if _com is a valid SerialConnection instance."""
         return Vedirect.is_serial_com(self._com)
 
     def is_serial_ready(self) -> bool:
@@ -97,18 +156,28 @@ class Vedirect:
         """Test if class Vedirect is ready"""
         return self.is_serial_ready()
 
-    def get_serial_port(self) -> str or None:
+    def get_serial_port(self) -> Optional[str]:
         """Test if class Vedirect is ready"""
         return self._com.get_serial_port()
 
     def connect_to_serial(self) -> bool:
-        """ Connect to serial port if not connected """
+        """
+        Connect to serial port if not connected.
+
+        Raise:
+         - SerialConfException: when invalid parameter is set.
+         - OpenSerialVeException: when serial device configured,
+           but connection not opened.
+         - SerialVeException: on raise serial.SerialException
+         - SerialConnectionException: on connection fails
+        """
         if self.has_serial_com()\
                 and ((not self._com.is_ready() and self._com.connect())
                      or self._com.is_ready()):
             result = True
+            time.sleep(0.5)
         else:
-            raise VedirectException(
+            raise SerialConnectionException(
                 "[Vedirect::init_serial_connection] "
                 "Connection to serial port fails."
             )
@@ -122,29 +191,35 @@ class Vedirect:
         Initialise serial connection from SerialConnection object.
 
         Raise:
-         - SettingInvalidException if serial_port, baud or timeout are not valid.
-         - VedirectException if connection to serial port fails
+         - SerialConfException: if serial_port, baud or timeout are not valid.
+         - OpenSerialVeException: when serial device configured,
+           but connection not opened.
+         - SerialVeException: on raise serial.SerialException
+         - SerialConnectionException: on connection fails
         :Example :
             >>> self.init_serial_connection_from_object(serial_connection)
             >>> True
-        :param self: Refer to the object itself,
-        :param serial_connection: The SerialConnection object,
-        :param auto_start: bool: Define if serial connection must be established automatically.
+        :param self:
+            Refer to the object itself,
+        :param serial_connection:
+            The SerialConnection object,
+        :param auto_start: bool:
+            Define if serial connection must be established automatically.
         :return: True if connection to serial port success.
         """
         result = False
-        if Vedirect.is_serial_com(serial_connection) and serial_connection.is_settings():
+        if Vedirect.is_serial_com(serial_connection)\
+                and serial_connection.is_settings():
             self._com = serial_connection
             if auto_start is False \
                     or (auto_start is True
                         and self.connect_to_serial()):
                 result = True
         else:
-            raise SettingInvalidException(
+            raise SerialConfException(
                 "[Vedirect::init_serial_connection_from_object] "
                 "Unable to init init_serial_connection_from_object, "
-                "bad parameters : %s" %
-                serial_connection
+                f"bad parameters : {serial_connection}"
             )
         return result
 
@@ -162,15 +237,22 @@ class Vedirect:
          - timeout = 0 (non blocking mode)
          - source_name = 'Vedirect'
         Raise:
-         - SettingInvalidException if serial_port, baud or timeout are not valid.
-         - VedirectException if connection to serial port fails
+         - SerialConfException: if serial_port, baud or timeout are not valid.
+         - OpenSerialVeException: when serial device configured,
+           but connection not opened.
+         - SerialVeException: on raise serial.SerialException
+         - SerialConnectionException: on connection fails
         :Example :
             >>> self.init_serial_connection({"serial_port": "/dev/ttyUSB1"})
             >>> True
-        :param self: Refer to the object itself,
-        :param serial_conf: dict: dict: The serial connection configuration,
-        :param source_name: str: This is used in logger to identify the source of call,
-        :param auto_start: bool: Define if serial connection must be established automatically.
+        :param self:
+            Refer to the object itself,
+        :param serial_conf: dict:
+            The serial connection configuration,
+        :param source_name: str:
+            This is used in logger to identify the source of call,
+        :param auto_start: bool:
+            Define if serial connection must be established automatically.
         :return: True if connection to serial port success.
         """
         result = False
@@ -184,7 +266,7 @@ class Vedirect:
                         and self.connect_to_serial()):
                 result = True
         else:
-            raise SettingInvalidException(
+            raise SerialConfException(
                 "[Vedirect::init_serial_connection] "
                 "Unable to init SerialConnection, "
                 "bad parameters. serial_conf : %s" %
@@ -201,21 +283,32 @@ class Vedirect:
         Initialise the settings for the class.
 
         At least serial_port must be provided.
+
         Default :
          - baud rate = 19200,
          - timeout = 0 (non blocking mode)
          - source_name = 'Vedirect'
-        Raise:
-         - SettingInvalidException if serial_port, baud or timeout are not valid.
-         - VedirectException if connection to serial port fails
+
         :Example :
             >>> self.init_settings({"serial_port": "/dev/ttyUSB1"})
             >>> True
-        :param self: Refer to the object itself,
-        :param serial_conf: dict: The serial connection configuration,
-        :param source_name: This is used in logger to identify the source of call,
-        :param auto_start: bool: Define if serial connection must be established automatically.
-        :return: True if connection to serial port success.
+        :param self:
+            Refer to the object itself,
+        :param serial_conf: dict:
+            The serial connection configuration,
+        :param source_name:
+            This is used in logger to identify the source of call,
+        :param auto_start: bool:
+            Define if serial connection must be established automatically.
+        :return:
+            True if connection to serial port success.
+
+        Raise:
+         - SerialConfException: if serial_port, baud or timeout are not valid.
+         - OpenSerialVeException: when serial device configured,
+           but connection not opened.
+         - SerialVeException: on raise serial.SerialException
+         - SerialConnectionException: on connection fails
         """
         return self.init_serial_connection(serial_conf=serial_conf,
                                            source_name=source_name,
@@ -230,23 +323,23 @@ class Vedirect:
         self.state = self.WAIT_HEADER
         self.dict = {}
 
-    def input_read(self, byte) -> dict or None:
+    def input_read(self, byte) -> Optional[dict]:
         """Input read from byte."""
         try:
-            nbyte = ord(byte)
-            if byte == self._delimiters.get("hexmarker")\
+            ord_byte = ord(byte)
+            if ord_byte == self._delimiters.get("hexmarker")\
                     and self.state != self.IN_CHECKSUM:
                 self.state = self.HEX
             if self.state == self.WAIT_HEADER:
-                self.bytes_sum += nbyte
-                if nbyte == self._delimiters.get("header1"):
+                self.bytes_sum += ord_byte
+                if ord_byte == self._delimiters.get("header1"):
                     self.state = self.WAIT_HEADER
-                elif nbyte == self._delimiters.get("header2"):
+                elif ord_byte == self._delimiters.get("header2"):
                     self.state = self.IN_KEY
                 return None
             elif self.state == self.IN_KEY:
-                self.bytes_sum += nbyte
-                if nbyte == self._delimiters.get("delimiter"):
+                self.bytes_sum += ord_byte
+                if ord_byte == self._delimiters.get("delimiter"):
                     if self.key == 'Checksum':
                         self.state = self.IN_CHECKSUM
                     else:
@@ -255,8 +348,10 @@ class Vedirect:
                     self.key += byte.decode('ascii')
                 return None
             elif self.state == self.IN_VALUE:
-                self.bytes_sum += nbyte
-                if nbyte == self._delimiters.get("header1"):
+                self.bytes_sum += ord_byte
+                if ord_byte == self._delimiters.get("header1"):
+                    if not self.has_free_block():
+                        raise PacketReadException
                     self.state = self.WAIT_HEADER
                     self.dict[self.key] = self.value
                     self.key = ''
@@ -265,7 +360,7 @@ class Vedirect:
                     self.value += byte.decode('ascii')
                 return None
             elif self.state == self.IN_CHECKSUM:
-                self.bytes_sum += nbyte
+                self.bytes_sum += ord_byte
                 self.key = ''
                 self.value = ''
                 self.state = self.WAIT_HEADER
@@ -276,31 +371,67 @@ class Vedirect:
                     self.bytes_sum = 0
             elif self.state == self.HEX:
                 self.bytes_sum = 0
-                if nbyte == self._delimiters.get("header2"):
+                if ord_byte == self._delimiters.get("header1"):
                     self.state = self.WAIT_HEADER
             else:
                 raise AssertionError()
+        except PacketReadException as ex:
+            raise PacketReadException(
+                "[Vedirect::input_read] "
+                "Serial input read error: "
+                "Packet has limit of "
+                f"{len(self.dict)}/{self.max_blocks} blocks"
+            ) from ex
         except Exception as ex:
             raise InputReadException(
                 "[Vedirect::input_read] "
-                "Serial input read error"
+                f"Serial input read error on byte : {byte}"
             ) from ex
 
-    def get_serial_packet(self) -> dict or None:
+    def get_serial_packet(self) -> Optional[dict]:
         """
         Return Ve Direct block packet from serial reader.
 
         Read a byte from serial and decode him with vedirect protocol.
-        :return: A dictionary of vedirect block data or None if block not entirely decoded.
+        :return:
+            A dict of vedirect block data
+            or None if block not entirely decoded.
         """
         byte = self._com.ser.read(1)
-        if byte == b'\x00':
-            byte = self._com.ser.read(1)
         return self.input_read(byte)
 
-    def read_data_single(self, timeout: int = 60) -> dict or None:
+    def read_global_packet(self, timeout: int = 60) -> Optional[dict]:
         """
-        Read a single block decoded from serial port and returns it as a dictionary.
+        Read Global packet from serial
+        """
+        run, now, tim = True, time.time(), 0
+        try:
+            while run:
+                packet, tim = None, time.time()
+
+                packet = self.get_serial_packet()
+
+                if packet is not None:
+                    logger.debug(
+                        "Serial reader success: dict: %s",
+                        packet
+                    )
+                    self.init_data_read()
+                    return packet
+
+                # timeout serial read
+                Vedirect.is_timeout(tim-now, timeout)
+        except SerialException as ex:
+            raise SerialVeException(
+                "[VeDirect:read_data_single] "
+                "Unable to read vedirect data."
+            ) from ex
+        return None
+
+    def read_data_single(self, timeout: int = 60) -> Optional[dict]:
+        """
+        Read a single block decoded from serial port
+        and returns it as a dictionary.
 
         :Example :
             >>> ve = Vedirect({"serial_port": "/dev/ttyUSB1"})
@@ -310,50 +441,41 @@ class Vedirect:
         :param self: Reference the class instance
         :param timeout: Set the timeout for the read_data_single function
         :return: A dictionary of the data
+        - SerialConfException:
+           Will be raised when parameter
+           are out of range or invalid,
+           e.g. serial_port, baud rate, data bits
+         - SerialVeException:
+           In case the device can not be found or can not be configured.
+         - OpenSerialVeException:
+           Will be raised when the device is configured
+           but port is not openned.
         """
-        run, now, tim = True, time.time(), 0
-        self.time_packet = 0
         if self.is_ready():
-            try:
-                while run:
-                    packet, tim = None, time.time()
-
-                    packet = self.get_serial_packet()
-
-                    if packet is not None:
-                        self.time_packet = tim
-                        logger.debug(
-                            "Serial reader success: dict: %s",
-                            self.dict
-                        )
-                        return packet
-
-                    # timeout serial read
-                    Vedirect.is_timeout(tim-now, timeout)
-            except SerialException as ex:
-                raise VedirectException(
-                    "[VeDirect:read_data_single] "
-                    "Unable to read vedirect data : %s.",
-                    ex
-                ) from SerialException
+            return self.read_global_packet(timeout)
         else:
-            logger.error('[VeDirect] Unable to read serial data. Not connected to serial port...')
-
-        return None
+            raise SerialConnectionException(
+                "[VeDirect:read_data_single] "
+                "Unable to read vedirect data, serial port is closed."
+            )
 
     def read_data_callback(self,
                            callback_function,
                            timeout: int = 60,
-                           max_loops: int or None = None
+                           max_loops: Optional[int] = None
                            ):
         """
         Read data from the serial port and returns it to a callback function.
 
-        :param self: Reference the class instance
-        :param callback_function:function: Pass a function to the read_data_callback function
-        :param timeout:int=60: Set the timeout for the read_data_callback function
-        :param max_loops:int or None=None: Limit the number of loops
+        :param self:
+            Reference the class instance
+        :param callback_function:function:
+            Pass a function to the read_data_callback function
+        :param timeout:int=60:
+            Set the timeout for the read_data_callback function
+        :param max_loops:Optional[int]=None: Limit the number of loops
         """
+        result = False
         run, now, tim, i = True, time.time(), 0, 0
         if self.is_ready():
             while run:
@@ -362,24 +484,30 @@ class Vedirect:
 
                 if packet is not None:
                     logger.debug(
-                        "Serial reader success: packet: %s "
-                        "-- state: %s -- bytes_sum: %s ",
+                        "Serial reader success: "
+                        "packet: %s -- "
+                        "state: %s -- "
+                        "bytes_sum: %s ",
                         packet, self.state, self.bytes_sum
                     )
-                    callback_function(packet)
                     now = tim
                     i = i + 1
+                    self.init_data_read()
+                    callback_function(packet)
 
                 # timeout serial read
                 Vedirect.is_timeout(tim-now, timeout)
                 if isinstance(max_loops, int) and 0 < max_loops <= i:
-                    return True
+                    result = True
+                    run = False
                 time.sleep(0.1)
         else:
-            raise VedirectException(
+            raise SerialConnectionException(
                 '[VeDirect::read_data_callback] '
                 'Unable to read serial data. '
-                'Not connected to serial port...')
+                'Not connected to serial port...'
+            )
+        return result
 
     @staticmethod
     def is_serial_com(obj: SerialConnection) -> bool:
@@ -392,10 +520,13 @@ class Vedirect:
         :param obj: The object to test.
         :return: True if obj is valid SerialConnection instance.
         """
-        return isinstance(obj, SerialConnection)
+        return isinstance(obj, SerialConnection)\
+            and obj.get_serial_port() is not None
 
     @staticmethod
-    def is_timeout(elapsed: float or int, timeout: float or int = 60) -> bool:
+    def is_timeout(elapsed: Union[int, float],
+                   timeout: Union[int, float] = 60
+                   ) -> bool:
         """
         Test if elapsed time is greater than timeout.
 
@@ -404,10 +535,10 @@ class Vedirect:
             >>> True
         :param elapsed: The elapsed time to test,
         :param timeout: The timeout to evaluate.
-        :return: True if elapsed time is upper than timeout.
+        :return: True if elapsed time is uppermore than timeout.
         """
         if elapsed >= timeout:
-            raise TimeoutException(
+            raise ReadTimeoutException(
                 '[VeDirect::is_timeout] '
                 'Unable to read serial data. '
                 'Timeout error.'
