@@ -13,16 +13,14 @@ import logging
 import inspect
 import os
 import time
+from typing import Optional
 import serial
 from ve_utils.utype import UType as Ut
 from ve_utils.utime import PerfStats
 
 __author__ = "Janne Kario, Eli Serra"
 __copyright__ = "Copyright 2015, Janne Kario"
-__deprecated__ = False
 __license__ = "MIT"
-__status__ = "Production"
-__version__ = "1.0.0"
 
 logging.basicConfig()
 logger = logging.getLogger("vedirect")
@@ -66,6 +64,18 @@ class Vedirectsim:
         """Test if is serial ready and connection opened."""
         return isinstance(self.ser, serial.Serial) and self.ser.is_open
 
+    def has_serial_connection(self) -> bool:
+        """Test if is serial ready and connection opened."""
+        if self.is_serial_ready():
+            logger.info(
+                "Connected to serial port %s",
+                self.serialport
+            )
+            return True
+        raise serial.SerialException(
+            f"Fatal error, unable to connect on serial port {self.serialport}."
+        )
+
     def serial_connect(self) -> bool:
         """Connect to serial port."""
         self.ser = serial.Serial(
@@ -75,17 +85,8 @@ class Vedirectsim:
             rtscts=False,
             dsrdtr=False
         )
-        self.ser.write_timeout = 0
-        if self.is_serial_ready():
-            logger.info(
-                "Connected to serial port %s",
-                self.serialport
-            )
-            return True
-        raise serial.SerialException(
-            "Fatal error, unable to connect on serial port %s." %
-            self.serialport
-        )
+        self.ser.write_timeout = 5
+        return self.has_serial_connection()
 
     def set_device(self, device: str):
         """Set the vedirect device to simulate."""
@@ -95,18 +96,47 @@ class Vedirectsim:
             return True
         return False
 
+    def get_dump_file_path(self) -> Optional[str]:
+        """Get dump files path if exist."""
+        result = None
+        current_script_path = os.path.dirname(
+            os.path.abspath(
+                inspect.getfile(inspect.currentframe())
+            )
+        )
+        path_order = [
+            os.path.join(current_script_path, "sim_data"),
+            os.path.join(os.path.abspath(os.getcwd()), "sim_data"),
+            os.path.join(os.path.abspath("/opt/vedirect_m8"), "sim_data"),
+            os.path.join(os.path.abspath(os.path.expanduser("~")), "sim_data"),
+        ]
+        file = f"{self.device}.dump"
+        for path in path_order:
+            current_path = os.path.join(path, file)
+            if os.path.isfile(current_path):
+                logger.debug(
+                    "Dump data loaded from %s",
+                    current_path
+                )
+                result = current_path
+                break
+
+        if result is None:
+            raise ValueError(
+                "Fatal error: "
+                f"Unable to locate dump file {file} from paths: {path_order}."
+            )
+        return result
+
     def set_dump_file_path(self):
         """Set the dump file path to read."""
-        if Ut.is_str(self.device) and self.device in Vedirectsim.get_valid_devices():
-            file = "%s.dump" % self.device
-            current_path = os.path.dirname(
-                os.path.abspath(
-                    inspect.getfile(inspect.currentframe())
-                    )
-                )
-            self.dump_file_path = os.path.join(current_path, "sim_data", file)
-            return os.path.isfile(self.dump_file_path)
-        return False
+        result = False
+        if Ut.is_str(self.device)\
+                and self.device in Vedirectsim.get_valid_devices():
+            self.dump_file_path = self.get_dump_file_path()
+            if self.dump_file_path is not None:
+                result = True
+        return result
 
     def set_device_settings(self, device: str) -> bool:
         """Set the device settings."""
@@ -118,9 +148,8 @@ class Vedirectsim:
             )
             return True
         raise ValueError(
-            "Fatal error, unable to set device settings on %s."
-            "Valid devices are : (bmv702, bluesolar_1.23, smartsolar_1.39) " %
-            device
+            f"Fatal error, unable to set device settings on {device}."
+            "Valid devices are : (bmv702, bluesolar_1.23, smartsolar_1.39) "
         )
 
     def convert(self, data: dict) -> list:
@@ -161,10 +190,9 @@ class Vedirectsim:
             self.perf.get_perf_key_stat("writes", "start")
         )
         sleep_time = 0
-        if 0 < write_time < 0.5:
-            sleep_time = round(0.5 - round(write_time, 3) - 0.0025, 3)
+        if 0 < write_time < 0.05:
+            sleep_time = abs(round(0.05 - round(write_time, 3) - 0.0025, 3))
             time.sleep(sleep_time)
-
         if logger.level <= 10:
             write_time = self.perf.get_elapsed(
                 self.perf.get_perf_key_stat("writes", "start")
@@ -182,7 +210,8 @@ class Vedirectsim:
                 self.dict
             )
 
-        self.dict = dict()
+        self.dict = {}
+        return packet_write
 
     def process_data(self, key: str, value: str):
         """Process read data."""
@@ -190,26 +219,26 @@ class Vedirectsim:
         if Ut.is_str(key, not_null=True):
             if key != 'Checksum':
                 self.dict.update({key: value})
-                if len(self.dict) == 18:
+                if len(self.dict) == 17:
                     result = True
             elif Ut.is_dict(self.dict, not_null=True):
                 result = True
         return result
 
-    def is_max_writes(self, max_writes: int or None = None):
+    def is_max_writes(self, max_writes: Optional[int] = None):
         """Test if max serial writes."""
         return Ut.is_int(max_writes) and self.block_counter >= max_writes
 
     def read_dump_file(self):
         """Read dump file."""
         if self.is_ready():
-            with open(self.dump_file_path) as lines:
+            with open(self.dump_file_path, encoding='utf-8') as lines:
                 for line in lines:
                     values = Vedirectsim.get_key_value_from_line(line)
                     if Ut.is_tuple(values, eq=2):
                         yield values
 
-    def read_dump_file_lines(self, max_writes: int or None = None) -> bool:
+    def read_dump_file_lines(self, max_writes: Optional[int] = None) -> bool:
         """Read file lines."""
         result = False
         if self.is_ready():
@@ -219,7 +248,8 @@ class Vedirectsim:
                 time.sleep(0.01)
                 if self.is_max_writes(max_writes):
                     logger.info(
-                        "End read dump file lines on max serial writes : %s/%s",
+                        "End read dump file lines on max serial writes : "
+                        "%s/%s",
                         self.block_counter, max_writes
                     )
                     result = True
@@ -232,7 +262,7 @@ class Vedirectsim:
 
         return result
 
-    def run(self, max_writes: int or None = None) -> bool:
+    def run(self, max_writes: Optional[int] = None) -> bool:
         """Run the simulator."""
         if self.is_ready():
             running = True
@@ -263,10 +293,9 @@ class Vedirectsim:
                     break
             return running
         raise ValueError(
-            "Fatal error, unable to set device settings on %s."
+            f"Fatal error, unable to set device settings on {self.device}."
             "Valid devices are : (bmv702, bluesolar_1.23, smartsolar_1.39). "
-            "Or can be path file error: %s" %
-            (self.device, self.dump_file_path)
+            f"Or can be path file error: {self.dump_file_path}"
         )
 
     @staticmethod
